@@ -1,4 +1,4 @@
-// Copyright (c) 2017 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -50,11 +50,15 @@ DEFINE_HIPRAND_STATE(hiprandStatePhilox4_32_10, rocrand_state_philox4x32_10)
 DEFINE_HIPRAND_STATE(hiprandStateMRG32k3a, rocrand_state_mrg32k3a)
 DEFINE_HIPRAND_STATE(hiprandStateMtgp32, rocrand_state_mtgp32)
 DEFINE_HIPRAND_STATE(hiprandStateSobol32, rocrand_state_sobol32)
+DEFINE_HIPRAND_STATE(hiprandStateScrambledSobol32, rocrand_state_scrambled_sobol32)
+DEFINE_HIPRAND_STATE(hiprandStateSobol64, rocrand_state_sobol64)
+DEFINE_HIPRAND_STATE(hiprandStateScrambledSobol64, rocrand_state_scrambled_sobol64)
 
 #undef DEFINE_HIPRAND_STATE
 
 typedef rocrand_discrete_distribution hiprandDiscreteDistribution_t;
-typedef unsigned int hiprandDirectionVectors32_t[32];
+typedef unsigned int                  hiprandDirectionVectors32_t[32];
+typedef unsigned long long int        hiprandDirectionVectors64_t[64];
 
 typedef mtgp32_params mtgp32_kernel_params_t;
 typedef mtgp32_fast_params mtgp32_fast_param_t;
@@ -121,18 +125,17 @@ template<typename StateType>
 QUALIFIERS
 void check_state_type()
 {
-    static_assert(
-        detail::is_any_of<
-            StateType,
-            hiprandState_t,
-            hiprandStateXORWOW_t,
-            hiprandStatePhilox4_32_10_t,
-            hiprandStateMRG32k3a_t,
-            hiprandStateMtgp32_t,
-            hiprandStateSobol32_t
-        >::value,
-        "StateType is not a hipRAND generator state"
-    );
+    static_assert(detail::is_any_of<StateType,
+                                    hiprandState_t,
+                                    hiprandStateXORWOW_t,
+                                    hiprandStatePhilox4_32_10_t,
+                                    hiprandStateMRG32k3a_t,
+                                    hiprandStateMtgp32_t,
+                                    hiprandStateSobol32_t,
+                                    hiprandStateScrambledSobol32_t,
+                                    hiprandStateSobol64_t,
+                                    hiprandStateScrambledSobol64_t>::value,
+                  "StateType is not a hipRAND generator state");
 }
 /// \endcond
 
@@ -215,36 +218,78 @@ void hiprand_init(const unsigned long long seed,
         "hiprandStateMtgp32_t does not have hiprand_init function, "
         "check hiprandMakeMTGP32KernelState() host function"
     );
-    static_assert(
-        !detail::is_any_of<
-            StateType,
-            hiprandStateSobol32_t
-        >::value,
-        "Quasirandom generators use different hiprand_init() function"
-    );
+    static_assert(!detail::is_any_of<StateType,
+                                     hiprandStateSobol32_t,
+                                     hiprandStateScrambledSobol32_t,
+                                     hiprandStateSobol64_t,
+                                     hiprandStateScrambledSobol64_t>::value,
+                  "Quasirandom generators use different hiprand_init() function");
     rocrand_init(seed, subsequence, offset, state);
 }
 
 /// \brief Initializes a Sobol32 state.
 ///
 /// \param direction_vectors - Pointer to array of 32 <tt>unsigned int</tt>s that
-/// represent the direction numbers
-/// \param offset - Absolute subsequence offset, i.e. how many states should be skipped
-/// \param state - Pointer to a state to initialize
-QUALIFIERS
-void hiprand_init(hiprandDirectionVectors32_t direction_vectors,
-                  unsigned int offset,
-                  hiprandStateSobol32_t * state)
+/// represent the direction numbers.
+/// \param offset - Absolute subsequence offset, i.e. how many states should be skipped.
+/// \param state - Pointer to a state to initialize.
+QUALIFIERS void hiprand_init(hiprandDirectionVectors32_t direction_vectors,
+                             unsigned int                offset,
+                             hiprandStateSobol32_t*      state)
 {
     rocrand_init(direction_vectors, offset, state);
 }
 
+/// \brief Initializes a ScrambledSobol32 state.
+///
+/// \param direction_vectors - Pointer to array of 32 <tt>unsigned int</tt>s that
+/// represent the direction numbers.
+/// \param scramble_constant - Constant used for scrambling the sequence.
+/// \param offset - Absolute subsequence offset, i.e. how many states should be skipped.
+/// \param state - Pointer to a state to initialize.
+QUALIFIERS void hiprand_init(hiprandDirectionVectors32_t     direction_vectors,
+                             unsigned int                    scramble_constant,
+                             unsigned int                    offset,
+                             hiprandStateScrambledSobol32_t* state)
+{
+    rocrand_init(direction_vectors, scramble_constant, offset, state);
+}
+
+/// \brief Initializes a Sobol64 state.
+///
+/// \param direction_vectors - Pointer to array of 64 <tt>unsigned long long int</tt>s that
+/// represent the direction numbers.
+/// \param offset - Absolute subsequence offset, i.e. how many states should be skipped.
+/// \param state - Pointer to a state to initialize.
+QUALIFIERS void hiprand_init(hiprandDirectionVectors64_t direction_vectors,
+                             unsigned int                offset,
+                             hiprandStateSobol64_t*      state)
+{
+    rocrand_init(direction_vectors, offset, state);
+}
+
+/// \brief Initializes a ScrambledSobol64 state.
+///
+/// \param direction_vectors - Pointer to array of 64 <tt>unsigned long long int</tt>s that
+/// represent the direction numbers.
+/// \param scramble_constant - Constant used for scrambling the sequence.
+/// \param offset - Absolute subsequence offset, i.e. how many states should be skipped.
+/// \param state - Pointer to a state to initialize.
+QUALIFIERS void hiprand_init(hiprandDirectionVectors64_t     direction_vectors,
+                             unsigned long long int          scramble_constant,
+                             unsigned int                    offset,
+                             hiprandStateScrambledSobol64_t* state)
+{
+    rocrand_init(direction_vectors, scramble_constant, offset, state);
+}
+
 /// \brief Updates RNG state skipping \p n states ahead.
 ///
 /// \tparam StateType - Random number generator state type.
 /// \p StateType type must be one of following types:
 /// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// or \p hiprandStateSobol32_t
+/// \p hiprandStateSobol32_t, \p hiprandStateScrambledSobol32_t,
+/// \p hiprandStateSobol64_t, or \p hiprandStateScrambledSobol64_t.
 ///
 /// \param n - Number of states to skipahead
 /// \param state - Pointer to a state to modify
@@ -284,14 +329,13 @@ QUALIFIERS
 void skipahead_sequence(unsigned long long n, StateType * state)
 {
     check_state_type<StateType>();
-    static_assert(
-        !detail::is_any_of<
-            StateType,
-            hiprandStateMtgp32_t,
-            hiprandStateSobol32_t
-        >::value,
-        "StateType does not have skipahead_sequence function"
-    );
+    static_assert(!detail::is_any_of<StateType,
+                                     hiprandStateMtgp32_t,
+                                     hiprandStateSobol32_t,
+                                     hiprandStateScrambledSobol32_t,
+                                     hiprandStateSobol64_t,
+                                     hiprandStateScrambledSobol64_t>::value,
+                  "StateType does not have skipahead_sequence function");
     typedef typename StateType::base base_type;
     skipahead_subsequence(n, static_cast<base_type*>(state));
 }
@@ -316,14 +360,13 @@ QUALIFIERS
 void skipahead_subsequence(unsigned long long n, StateType * state)
 {
     check_state_type<StateType>();
-    static_assert(
-        !detail::is_any_of<
-            StateType,
-            hiprandStateMtgp32_t,
-            hiprandStateSobol32_t
-        >::value,
-        "StateType does not have skipahead_subsequence function"
-    );
+    static_assert(!detail::is_any_of<StateType,
+                                     hiprandStateMtgp32_t,
+                                     hiprandStateSobol32_t,
+                                     hiprandStateScrambledSobol32_t,
+                                     hiprandStateSobol64_t,
+                                     hiprandStateScrambledSobol64_t>::value,
+                  "StateType does not have skipahead_subsequence function");
     typedef typename StateType::base base_type;
     skipahead_subsequence(n, static_cast<base_type*>(state));
 }
@@ -334,7 +377,7 @@ void skipahead_subsequence(unsigned long long n, StateType * state)
 /// \tparam StateType - Random number generator state type.
 /// \p StateType type must be one of following types:
 /// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
+/// \p hiprandStateMtgp32_t, \p hiprandStateSobol32_t, \p hiprandStateScrambledSobol32_t.
 ///
 /// \param state - Pointer to a RNG state to use
 /// \return Uniformly distributed random 32-bit <tt>unsigned int</tt>
@@ -357,13 +400,26 @@ uint4 hiprand4(hiprandStatePhilox4_32_10_t * state)
     return rocrand4(state);
 }
 
+/// \brief Generates uniformly distributed random <tt>unsigned long long int</tt>
+/// from [0; 2^64 - 1] range.
+///
+/// \tparam StateType - Random number generator state type.
+/// \p StateType type must be one of the following types:
+/// \p hiprandStateSobol64_t or \p hiprandStateScrambledSobol64_t.
+///
+/// \param state - Pointer to a RNG state to use
+/// \return Uniformly distributed random 64-bit <tt>unsigned long long int</tt>
+template<class StateType>
+QUALIFIERS unsigned long long int hiprand_long_long(StateType* state)
+{
+    check_state_type<StateType>();
+    return rocrand(state);
+}
+
 /// \brief Generates uniformly distributed random <tt>float</tt> value
 /// from (0; 1] range.
 ///
 /// \tparam StateType - Random number generator state type.
-/// \p StateType type must be one of following types:
-/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
 ///
 /// \param state - Pointer to a RNG state to use
 /// \return Uniformly distributed random <tt>float</tt> value
@@ -389,18 +445,14 @@ float4 hiprand_uniform4(hiprandStatePhilox4_32_10_t * state)
 /// \brief Generates uniformly distributed random <tt>double</tt> value from (0; 1] range
 ///
 /// \tparam StateType - Random number generator state type.
-/// \p StateType type must be one of following types:
-/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
 ///
 /// \param state - Pointer to a RNG state to use
 /// \return Uniformly distributed random <tt>double</tt> value
 ///
 /// Note: When \p state is of type: \p hiprandStateMRG32k3a_t, \p hiprandStateMtgp32_t,
-/// or \p hiprandStateSobol32_t, then the returned \p double value is generated
-/// using only 32 random bits (one <tt>unsigned int</tt> value).
-/// In case of type \p hiprandStateSobol32_t, this is done to guarantee the quasirandom
-/// properties of the Sobol32 sequence.
+/// \p hiprandStateSobol32_t, or \p hiprandStateScrambledSobol32_t then the returned
+/// \p double value is generated using only 32 random bits (one <tt>unsigned int</tt> value).
+/// In case of the Sobol types, this is done to guarantee the quasirandom properties.
 template<class StateType>
 QUALIFIERS
 double hiprand_uniform_double(StateType * state)
@@ -437,10 +489,7 @@ double4 hiprand_uniform4_double(hiprandStatePhilox4_32_10_t * state)
 /// equals 1.0.
 ///
 /// \tparam StateType - Random number generator state type.
-/// \p StateType type must be one of following types:
-/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
-///
+
 /// \param state - Pointer to a RNG state to use
 /// \return Normally distributed random <tt>float</tt> value
 template<class StateType>
@@ -500,9 +549,6 @@ float4 hiprand_normal4(hiprandStatePhilox4_32_10_t * state)
 /// equals 1.0.
 ///
 /// \tparam StateType - Random number generator state type.
-/// \p StateType type must be one of following types:
-/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
 ///
 /// \param state - Pointer to a RNG state to use
 /// \return Normally distributed random <tt>double</tt> value
@@ -560,9 +606,6 @@ double4 hiprand_normal4_double(hiprandStatePhilox4_32_10_t * state)
 /// \brief Generates log-normally distributed random <tt>float</tt> value
 ///
 /// \tparam StateType - Random number generator state type.
-/// \p StateType type must be one of following types:
-/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
 ///
 /// \param state - Pointer to a RNG state to use
 /// \param mean - Mean value of log-normal distribution
@@ -623,9 +666,6 @@ float4 hiprand_log_normal4(hiprandStatePhilox4_32_10_t * state,
 /// \brief Generates log-normally distributed random <tt>double</tt> value
 ///
 /// \tparam StateType - Random number generator state type.
-/// \p StateType type must be one of following types:
-/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
 ///
 /// \param state - Pointer to a RNG state to use
 /// \param mean - Mean value of log-normal distribution
@@ -644,8 +684,8 @@ double hiprand_log_normal_double(StateType * state,
 ///
 /// \tparam StateType - Random number generator state type.
 /// \p StateType type must be one of following types:
-/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
+/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t,
+/// \p hiprandStateMRG32k3a_t, or \p hiprandStateMtgp32_t.
 ///
 /// \param state - Pointer to a RNG state to use
 /// \param mean - Mean value of log-normal distribution
@@ -686,9 +726,6 @@ double4 hiprand_log_normal4_double(hiprandStatePhilox4_32_10_t * state,
 /// \brief Generates Poisson-distributed random <tt>unsigned int</tt> value
 ///
 /// \tparam StateType - Random number generator state type.
-/// \p StateType type must be one of following types:
-/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
 ///
 /// \param state - Pointer to a RNG state to use
 /// \param lambda - Lambda (mean) parameter of Poisson distribution
@@ -716,9 +753,6 @@ uint4 hiprand_poisson4(hiprandStatePhilox4_32_10_t * state, double lambda)
 /// given discrete distribution
 ///
 /// \tparam StateType - Random number generator state type.
-/// \p StateType type must be one of following types:
-/// \p hiprandStateXORWOW_t, \p hiprandStatePhilox4_32_10_t, \p hiprandStateMRG32k3a_t,
-/// \p hiprandStateMtgp32_t, or \p hiprandStateSobol32_t
 ///
 /// \param state - Pointer to a RNG state to use
 /// \param discrete_distribution - Discrete distribution
